math.randomseed(os.time())

local lg = love.graphics
local lk = love.keyboard

local cron = require "lib.cron"
local inspect = require "lib.inspect"

local Spawn = require "EnemySpawns"
local Bullet = require "Bullet"

local enemies = {}
local bullets = {}

Spawn.CreateEnemyGroup(bullets, enemies, false, "randomSpeed", "random", "random")

local enemySpawner = cron.every(1, function() Spawn.CreateEnemyGroup(bullets, enemies, false, "randomSpeed", "random", "random") end)

local function outOfBounds(e)
    -- I decided to only check if gone far enough left, regardless of entry / exit,
    --  everything once it reaches the far left is dead

    --if e.x < e.w / 2 or e.y < e.h / 2 or e.y > lg.getHeight() + e.h / 2 then
    if e.x < e.w / 2 then
        return true
    end
end

local player = {
    x = 10,
    y = lg.getHeight() / 2,
    v = {
        x = 0,
        y = 0
    },
    w = 10,
    h = 10,
    color = {0, 255, 0},
    speed = 540/3
}

local bulletFiring
local function fireBullet()
    if lk.isDown(' ') then
        table.insert(bullets, Bullet(player.x, player.y))--, false, false, player.v.x * 20 + player.speed * 2))
        bulletFiring = cron.after(1/12, fireBullet)
    end
end

local function checkAABB(A, B)
    if not A or not B then return end -- because fuck everything
    return A.x - A.w / 2 < B.x + B.w / 2 and B.x - B.w / 2 < A.x + A.w / 2 and A.y - A.h / 2 < B.y + B.h / 2 and B.y - B.h / 2 < A.y + A.h / 2
    --[[
    this.x <= rect.x + rect.width &&
                rect.x <= this.x + this.width &&
                this.y <= rect.y + rect.height &&
                rect.y <= this.y + this.height);
    ]]
end

function love.update(dt)
    enemySpawner:update(dt)
    for i=#bullets,1,-1 do
        bullets[i]:update(dt)
        if bullets[i].x > lg.getWidth() + bullets[i].w / 2 then
            table.remove(bullets, i)
            i = i + 1
        end
    end
    for i=#enemies,1,-1 do
        enemies[i]:update(dt)
        --e.x < e.w / 2
        if enemies[i].x < enemies[i].w / 2 then
            table.remove(enemies, i)
            i = i + 1 --so we don't skip one!
        end
    end
    if lk.isDown('w') then
        player.v.y = player.v.y - player.speed * dt
    elseif lk.isDown('s') then
        player.v.y = player.v.y + player.speed * dt
    end
    --[[
    if lk.isDown('a') then
        player.v.x = player.v.x - player.speed * dt
    elseif lk.isDown('d') then
        player.v.x = player.v.x + player.speed * dt
    end
    --]]
    player.v.x = player.v.x * 0.9
    player.v.y = player.v.y * 0.9
    player.x = player.x + player.v.x
    player.y = player.y + player.v.y
    if player.x < player.w / 2 then
        player.x = player.w / 2
        player.v.x = 0
    elseif player.x > lg.getWidth() - player.w / 2 then
        player.x = lg.getWidth() - player.w / 2
        player.v.x = 0
    end
    if player.y < player.h / 2 then
        player.y = player.h / 2
        player.v.y = 0
    elseif player.y > lg.getHeight() - player.h / 2 then
        player.y = lg.getHeight() - player.h / 2
        player.v.y = 0
    end

    if bulletFiring then bulletFiring:update(dt) end

    --check collisions with bullets and enemies
    --checkAABB
    for i=#enemies,1,-1 do
        for j=#bullets,1,-1 do
            if checkAABB(enemies[i], bullets[j]) then
                table.remove(enemies, i)
                table.remove(bullets, j)
                -- fuck it, if it works with duplicated checks, I don't care
                --i = i + 1
                --j = j + 1
                --print(i, #enemies, j, #bullets)
            end
        end
    end
end

function love.draw()
    for i=1,#bullets do
        lg.setColor(bullets[i].color)
        lg.rectangle("fill", bullets[i].x - bullets[i].w / 2, bullets[i].y - bullets[i].h / 2, bullets[i].w, bullets[i].h)
    end
    for i=1,#enemies do
        lg.setColor(enemies[i].color)
        lg.rectangle("fill", enemies[i].x - enemies[i].w / 2, enemies[i].y - enemies[i].h / 2, enemies[i].w, enemies[i].h)
    end
    lg.setColor(player.color)
    lg.rectangle("fill", player.x - player.w / 2, player.y - player.h / 2, player.w, player.h)
end

function love.keypressed(key, unicode)
    if key == 'escape' then
        love.event.quit()
    end

    fireBullet()
end
